@@ -24,11 +24,11 @@ namespace OpenBve {
 				this.TrackPosition = TrackPosition;
 			}
 		}
-		internal static Fog PreviousFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 0.0);
-		internal static Fog CurrentFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 0.5);
-		internal static Fog NextFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 1.0);
-		internal static float NoFogStart = 800.0f;
+		internal static float NoFogStart = 800.0f; // must not be 600 nor below
 		internal static float NoFogEnd = 1600.0f;
+		internal static Fog PreviousFog = new Fog(NoFogStart, NoFogEnd, new World.ColorRGB(128, 128, 128), 0.0);
+		internal static Fog CurrentFog = new Fog(NoFogStart, NoFogEnd, new World.ColorRGB(128, 128, 128), 0.5);
+		internal static Fog NextFog = new Fog(NoFogStart, NoFogEnd, new World.ColorRGB(128, 128, 128), 1.0);
 		
 		// route constants
 		internal static string RouteComment = "";
@@ -40,6 +40,7 @@ namespace OpenBve {
 		internal static double RouteInitialElevation = 0.0;
 		internal static double RouteSeaLevelAirPressure = 101325.0;
 		internal static double RouteSeaLevelAirTemperature = 293.15;
+		internal static double[] RouteUnitOfLength = new double[] { 1.0 };
 		internal const double CoefficientOfGroundFriction = 0.5;
 		internal const double CriticalCollisionSpeedDifference = 8.0;
 		internal const double BrakePipeLeakRate = 500000.0;
@@ -100,15 +101,16 @@ namespace OpenBve {
 
 		// other trains
 		internal static double[] PrecedingTrainTimeDeltas = new double[] { };
+		internal static double PrecedingTrainSpeedLimit = double.PositiveInfinity;
 		internal static BogusPretrainInstruction[] BogusPretrainInstructions = new BogusPretrainInstruction[] { };
 
 		// startup
 		internal enum TrainStartMode {
 			ServiceBrakesAts = -1,
-			EmergencyBrakesNoAts = 0,
-			EmergencyBrakesAts = 1
+			EmergencyBrakesAts = 0,
+			EmergencyBrakesNoAts = 1
 		}
-		internal static TrainStartMode TrainStart = TrainStartMode.EmergencyBrakesNoAts;
+		internal static TrainStartMode TrainStart = TrainStartMode.EmergencyBrakesAts;
 		internal static string TrainName = "";
 
 		// information
@@ -247,14 +249,15 @@ namespace OpenBve {
 			MarkerTextures = new int[] { };
 			PointsOfInterest = new PointOfInterest[] { };
 			PrecedingTrainTimeDeltas = new double[] { };
+			PrecedingTrainSpeedLimit = double.PositiveInfinity;
 			BogusPretrainInstructions = new BogusPretrainInstruction[] { };
 			TrainName = "";
 			TrainStart = TrainStartMode.EmergencyBrakesNoAts;
-			PreviousFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 0.0);
-			CurrentFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 0.5);
-			NextFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 1.0);
-			NoFogStart = (float)World.BackgroundImageDistance + 200.0f;
-			NoFogEnd = 2.0f * NoFogStart;
+			NoFogStart = (float)Math.Max(1.33333333333333 * World.BackgroundImageDistance, 800.0);
+			NoFogEnd = (float)Math.Max(2.66666666666667 * World.BackgroundImageDistance, 1600.0);
+			PreviousFog = new Fog(NoFogStart, NoFogEnd, new World.ColorRGB(128, 128, 128), 0.0);
+			CurrentFog = new Fog(NoFogStart, NoFogEnd, new World.ColorRGB(128, 128, 128), 0.5);
+			NextFog = new Fog(NoFogStart, NoFogEnd, new World.ColorRGB(128, 128, 128), 1.0);
 			InfoTotalTriangles = 0;
 			InfoTotalTriangleStrip = 0;
 			InfoTotalQuads = 0;
@@ -310,7 +313,7 @@ namespace OpenBve {
 				bool leftopen = false;
 				bool rightopen = false;
 				for (int j = 0; j < TrainManager.PlayerTrain.Cars.Length; j++) {
-					int k; for (k = 0; k < TrainManager.PlayerTrain.Cars[j].Specs.Doors.Length; k++) {
+					for (int k = 0; k < TrainManager.PlayerTrain.Cars[j].Specs.Doors.Length; k++) {
 						if (TrainManager.PlayerTrain.Cars[j].Specs.Doors[k].State != 0.0) {
 							if (TrainManager.PlayerTrain.Cars[j].Specs.Doors[k].Direction == -1) {
 								leftopen = true;
@@ -349,7 +352,7 @@ namespace OpenBve {
 				}
 			}
 			// overspeed
-			if (TrainManager.PlayerTrain.Specs.Security.Mode != TrainManager.SecuritySystem.Atc) {
+			if (TrainManager.PlayerTrain.Specs.Safety.Mode != TrainManager.SafetySystem.Atc) {
 				double nr = TrainManager.PlayerTrain.CurrentRouteLimit;
 				double ns = TrainManager.PlayerTrain.CurrentSectionLimit;
 				double n = nr < ns ? nr : ns;
@@ -406,7 +409,7 @@ namespace OpenBve {
 			}
 			// red signal
 			{
-				if (TrainManager.PlayerTrain.CurrentSectionLimit == 0.0 & TrainManager.PlayerTrain.Specs.Security.Mode != TrainManager.SecuritySystem.Atc) {
+				if (TrainManager.PlayerTrain.CurrentSectionLimit == 0.0 & TrainManager.PlayerTrain.Specs.Safety.Mode != TrainManager.SafetySystem.Atc) {
 					if (!CurrentScore.RedSignal) {
 						int x = ScoreValueRedSignal;
 						CurrentScore.Value += x;
@@ -657,11 +660,11 @@ namespace OpenBve {
 			internal float Speed;
 			internal float Acceleration;
 			internal short ReverserDriver;
-			internal short ReverserSecurity;
+			internal short ReverserSafety;
 			internal BlackBoxPower PowerDriver;
-			internal BlackBoxPower PowerSecurity;
+			internal BlackBoxPower PowerSafety;
 			internal BlackBoxBrake BrakeDriver;
-			internal BlackBoxBrake BrakeSecurity;
+			internal BlackBoxBrake BrakeSafety;
 			internal BlackBoxEventToken EventToken;
 		}
 		internal static BlackBoxEntry[] BlackBoxEntries = new BlackBoxEntry[256];
@@ -684,9 +687,9 @@ namespace OpenBve {
 				BlackBoxEntries[BlackBoxEntryCount].Speed = (float)TrainManager.PlayerTrain.Specs.CurrentAverageSpeed;
 				BlackBoxEntries[BlackBoxEntryCount].Acceleration = (float)TrainManager.PlayerTrain.Specs.CurrentAverageAcceleration;
 				BlackBoxEntries[BlackBoxEntryCount].ReverserDriver = (short)TrainManager.PlayerTrain.Specs.CurrentReverser.Driver;
-				BlackBoxEntries[BlackBoxEntryCount].ReverserSecurity = (short)TrainManager.PlayerTrain.Specs.CurrentReverser.Actual;
+				BlackBoxEntries[BlackBoxEntryCount].ReverserSafety = (short)TrainManager.PlayerTrain.Specs.CurrentReverser.Actual;
 				BlackBoxEntries[BlackBoxEntryCount].PowerDriver = (BlackBoxPower)TrainManager.PlayerTrain.Specs.CurrentPowerNotch.Driver;
-				BlackBoxEntries[BlackBoxEntryCount].PowerSecurity = (BlackBoxPower)TrainManager.PlayerTrain.Specs.CurrentPowerNotch.Security;
+				BlackBoxEntries[BlackBoxEntryCount].PowerSafety = (BlackBoxPower)TrainManager.PlayerTrain.Specs.CurrentPowerNotch.Safety;
 				if (TrainManager.PlayerTrain.Specs.CurrentEmergencyBrake.Driver) {
 					BlackBoxEntries[BlackBoxEntryCount].BrakeDriver = BlackBoxBrake.Emergency;
 				} else if (TrainManager.PlayerTrain.Specs.CurrentHoldBrake.Driver) {
@@ -701,19 +704,19 @@ namespace OpenBve {
 				} else {
 					BlackBoxEntries[BlackBoxEntryCount].BrakeDriver = (BlackBoxBrake)TrainManager.PlayerTrain.Specs.CurrentBrakeNotch.Driver;
 				}
-				if (TrainManager.PlayerTrain.Specs.CurrentEmergencyBrake.Security) {
-					BlackBoxEntries[BlackBoxEntryCount].BrakeSecurity = BlackBoxBrake.Emergency;
+				if (TrainManager.PlayerTrain.Specs.CurrentEmergencyBrake.Safety) {
+					BlackBoxEntries[BlackBoxEntryCount].BrakeSafety = BlackBoxBrake.Emergency;
 				} else if (TrainManager.PlayerTrain.Specs.CurrentHoldBrake.Actual) {
-					BlackBoxEntries[BlackBoxEntryCount].BrakeSecurity = BlackBoxBrake.HoldBrake;
+					BlackBoxEntries[BlackBoxEntryCount].BrakeSafety = BlackBoxBrake.HoldBrake;
 				} else if (TrainManager.PlayerTrain.Cars[d].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake) {
-					switch (TrainManager.PlayerTrain.Specs.AirBrake.Handle.Security) {
-							case TrainManager.AirBrakeHandleState.Release: BlackBoxEntries[BlackBoxEntryCount].BrakeSecurity = BlackBoxBrake.Release; break;
-							case TrainManager.AirBrakeHandleState.Lap: BlackBoxEntries[BlackBoxEntryCount].BrakeSecurity = BlackBoxBrake.Lap; break;
-							case TrainManager.AirBrakeHandleState.Service: BlackBoxEntries[BlackBoxEntryCount].BrakeSecurity = BlackBoxBrake.Service; break;
-							default: BlackBoxEntries[BlackBoxEntryCount].BrakeSecurity = BlackBoxBrake.Emergency; break;
+					switch (TrainManager.PlayerTrain.Specs.AirBrake.Handle.Safety) {
+							case TrainManager.AirBrakeHandleState.Release: BlackBoxEntries[BlackBoxEntryCount].BrakeSafety = BlackBoxBrake.Release; break;
+							case TrainManager.AirBrakeHandleState.Lap: BlackBoxEntries[BlackBoxEntryCount].BrakeSafety = BlackBoxBrake.Lap; break;
+							case TrainManager.AirBrakeHandleState.Service: BlackBoxEntries[BlackBoxEntryCount].BrakeSafety = BlackBoxBrake.Service; break;
+							default: BlackBoxEntries[BlackBoxEntryCount].BrakeSafety = BlackBoxBrake.Emergency; break;
 					}
 				} else {
-					BlackBoxEntries[BlackBoxEntryCount].BrakeSecurity = (BlackBoxBrake)TrainManager.PlayerTrain.Specs.CurrentBrakeNotch.Security;
+					BlackBoxEntries[BlackBoxEntryCount].BrakeSafety = (BlackBoxBrake)TrainManager.PlayerTrain.Specs.CurrentBrakeNotch.Safety;
 				}
 				BlackBoxEntries[BlackBoxEntryCount].EventToken = EventToken;
 				BlackBoxEntryCount++;
@@ -729,7 +732,7 @@ namespace OpenBve {
 			internal double BackwardTolerance;
 			internal int Cars;
 		}
-		internal enum SecuritySystem {
+		internal enum SafetySystem {
 			Any = -1,
 			Ats = 0,
 			Atc = 1
@@ -753,11 +756,12 @@ namespace OpenBve {
 			internal bool ForceStopSignal;
 			internal bool OpenLeftDoors;
 			internal bool OpenRightDoors;
-			internal SecuritySystem SecuritySystem;
+			internal SafetySystem SafetySystem;
 			internal StationStop[] Stops;
 			internal double PassengerRatio;
 			internal int TimetableDaytimeTexture;
 			internal int TimetableNighttimeTexture;
+			internal double DefaultTrackPosition;
 		}
 		internal static Station[] Stations = new Station[] { };
 		internal static int GetStopIndex(int StationIndex, int Cars) {
@@ -867,13 +871,11 @@ namespace OpenBve {
 			bool settored = false;
 			if (Sections[SectionIndex].Type == SectionType.ValueBased) {
 				// value-based
-				zeroaspect = int.MaxValue;
-				for (int i = 0; i < Sections[SectionIndex].Aspects.Length; i++) {
-					if (Sections[SectionIndex].Aspects[i].Number < zeroaspect) {
-						zeroaspect = Sections[SectionIndex].Aspects[i].Number;
+				zeroaspect = 0;
+				for (int i = 1; i < Sections[SectionIndex].Aspects.Length; i++) {
+					if (Sections[SectionIndex].Aspects[i].Number < Sections[SectionIndex].Aspects[zeroaspect].Number) {
+						zeroaspect = i;
 					}
-				} if (zeroaspect == int.MaxValue) {
-					zeroaspect = -1;
 				}
 			} else {
 				// index-based
@@ -969,13 +971,15 @@ namespace OpenBve {
 					int n = Sections[SectionIndex].NextSection;
 					int a = Sections[SectionIndex].Aspects[Sections[SectionIndex].Aspects.Length - 1].Number;
 					if (n >= 0 && Sections[n].CurrentAspect >= 0) {
+						
 						a = Sections[n].Aspects[Sections[n].CurrentAspect].Number;
 					}
 					for (int i = Sections[SectionIndex].Aspects.Length - 1; i >= 0; i--) {
 						if (Sections[SectionIndex].Aspects[i].Number > a) {
 							newaspect = i;
 						}
-					} if (newaspect == -1) {
+					}
+					if (newaspect == -1) {
 						newaspect = Sections[SectionIndex].Aspects.Length - 1;
 					}
 				} else {
@@ -987,7 +991,7 @@ namespace OpenBve {
 					}
 				}
 			}
-			if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) {
+			if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) {
 				// call setsignal for plugins when section changes aspect
 				int p = Sections[SectionIndex].PreviousSection;
 				if (p >= 0 && Sections[p].Trains.Length == 1 && Sections[p].Trains[0] == TrainManager.PlayerTrain && TrainManager.PlayerTrain.CurrentSectionIndex == p) {
@@ -1020,11 +1024,12 @@ namespace OpenBve {
 
 		// ai
 		internal abstract class GeneralAI {
-			internal abstract void Trigger(TrainManager.Train Train);
+			internal abstract void Trigger(TrainManager.Train Train, double TimeElapsed);
 		}
-
-		// simplistic human driver
-		internal class SimplisticHumanDriverAI : GeneralAI {
+		
+		// simple human driver
+		internal class SimpleHumanDriverAI : GeneralAI {
+			// members
 			private double TimeLastProcessed;
 			private double CurrentInterval;
 			private bool BrakeMode;
@@ -1032,16 +1037,17 @@ namespace OpenBve {
 			private int PowerNotchAtWhichWheelSlipIsObserved;
 			private double AtsChimeCancelPosition;
 			private int AtsChimeCancelSection;
-			internal SimplisticHumanDriverAI(TrainManager.Train Train) {
+			// functions
+			internal SimpleHumanDriverAI(TrainManager.Train Train) {
 				this.TimeLastProcessed = 0.0;
 				this.CurrentInterval = 1.0;
 				this.BrakeMode = false;
-				this.PersonalitySpeedFactor = 0.92 + 0.08 * Generator.NextDouble();
+				this.PersonalitySpeedFactor = 0.9 + 0.1 * Generator.NextDouble();
 				this.PowerNotchAtWhichWheelSlipIsObserved = Train.Specs.MaximumPowerNotch + 1;
 				this.AtsChimeCancelPosition = Train.Cars[0].FrontAxle.Follower.TrackPosition + 600.0;
 				this.AtsChimeCancelSection = -1;
 			}
-			internal override void Trigger(TrainManager.Train Train) {
+			internal override void Trigger(TrainManager.Train Train, double TimeElapsed) {
 				if (SecondsSinceMidnight - TimeLastProcessed >= CurrentInterval) {
 					TimeLastProcessed = SecondsSinceMidnight;
 					// door states
@@ -1055,18 +1061,18 @@ namespace OpenBve {
 							if (doorsopen) break;
 						}
 					}
-					// handle the security system
-					if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.None) {
+					// handle the safety system
+					if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.None) {
 						// none
-						Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.AtsSN;
+						Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.AtsSn;
 						this.AtsChimeCancelPosition = Train.Cars[0].FrontAxle.Follower.TrackPosition + 600.0;
 						this.AtsChimeCancelSection = -1;
-					} else if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.AtsSN) {
+					} else if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.AtsSn) {
 						// ats-s
-						if (Train.Specs.Security.State == TrainManager.SecurityState.Ringing) {
+						if (Train.Specs.Safety.State == TrainManager.SafetyState.Ringing) {
 							if (Train.Specs.CurrentPowerNotch.Driver == 0 & (Train.Specs.CurrentBrakeNotch.Driver >= 1 & Train.Cars[Train.DriverCar].Specs.BrakeType != TrainManager.CarBrakeType.AutomaticAirBrake | Train.Specs.AirBrake.Handle.Driver == TrainManager.AirBrakeHandleState.Service & Train.Cars[Train.DriverCar].Specs.BrakeType != TrainManager.CarBrakeType.AutomaticAirBrake) | Train.Specs.CurrentEmergencyBrake.Driver) {
 								// acknowledge bell
-								TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Alarm);
+								TrainManager.AcknowledgeSafetySystem(Train, TrainManager.AcknowledgementType.Alarm);
 								// keep track of next red signal
 								const double lookahead = 700.0;
 								double tp = Train.Cars[0].FrontAxle.Follower.TrackPosition - Train.Cars[0].FrontAxlePosition + 0.5 * Train.Cars[0].Length;
@@ -1077,7 +1083,7 @@ namespace OpenBve {
 										if (tp + lookahead <= stp) break;
 										for (int j = 0; j < TrackManager.CurrentTrack.Elements[i].Events.Length; j++) {
 											if (TrackManager.CurrentTrack.Elements[i].Events[j] is TrackManager.SectionChangeEvent) {
-												if (Train.Specs.Security.Mode != TrainManager.SecuritySystem.Atc) {
+												if (Train.Specs.Safety.Mode != TrainManager.SafetySystem.Atc) {
 													TrackManager.SectionChangeEvent e = (TrackManager.SectionChangeEvent)TrackManager.CurrentTrack.Elements[i].Events[j];
 													if (stp + e.TrackPositionDelta > tp) {
 														if (!Game.Sections[e.NextSectionIndex].Invisible & Game.Sections[e.NextSectionIndex].CurrentAspect >= 0) {
@@ -1100,54 +1106,54 @@ namespace OpenBve {
 									this.AtsChimeCancelSection = -1;
 								}
 							}
-						} else if (Train.Specs.Security.State == TrainManager.SecurityState.Emergency & Math.Abs(Train.Specs.CurrentAverageSpeed) < 1.0) {
+						} else if (Train.Specs.Safety.State == TrainManager.SafetyState.Emergency & Math.Abs(Train.Specs.CurrentAverageSpeed) < 1.0) {
 							// reset the safety system
-							Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.None;
+							Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.None;
 							this.CurrentInterval = 1.0;
 							return;
 						}
 						// cancel chime
 						if (Train.Cars[0].FrontAxle.Follower.TrackPosition > this.AtsChimeCancelPosition) {
-							TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Chime);
+							TrainManager.AcknowledgeSafetySystem(Train, TrainManager.AcknowledgementType.Chime);
 							this.AtsChimeCancelPosition = double.PositiveInfinity;
 							this.AtsChimeCancelSection = -1;
 						} else if (this.AtsChimeCancelSection >= 0 && Game.Sections[this.AtsChimeCancelSection].Aspects[Game.Sections[this.AtsChimeCancelSection].CurrentAspect].Speed != 0.0) {
-							TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Chime);
+							TrainManager.AcknowledgeSafetySystem(Train, TrainManager.AcknowledgementType.Chime);
 							this.AtsChimeCancelPosition = double.PositiveInfinity;
 							this.AtsChimeCancelSection = -1;
 						}
-					} else if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.AtsP) {
+					} else if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.AtsP) {
 						// ats-p
-						TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Chime);
+						TrainManager.AcknowledgeSafetySystem(Train, TrainManager.AcknowledgementType.Chime);
 						this.AtsChimeCancelPosition = double.PositiveInfinity;
 						this.AtsChimeCancelSection = -1;
-						if (Train.Specs.Security.State == TrainManager.SecurityState.Service & !Train.Specs.Security.Ats.AtsPOverride) {
-							if (Train.Specs.Security.Ats.AtsPDistance > 50.0 | Train.Station >= 0) {
+						if (Train.Specs.Safety.State == TrainManager.SafetyState.Service & !Train.Specs.Safety.Ats.AtsPOverride) {
+							if (Train.Specs.Safety.Ats.AtsPDistance > 50.0 | Train.Station >= 0) {
 								if (Math.Abs(Train.Specs.CurrentAverageSpeed) < 4.16666666666667) {
-									TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Override);
+									TrainManager.AcknowledgeSafetySystem(Train, TrainManager.AcknowledgementType.Override);
 									return;
 								}
 							} else {
 								if (Math.Abs(Train.Specs.CurrentAverageSpeed) < 0.277777777777778) {
-									Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.None;
+									Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.None;
 									return;
 								}
 							}
 						}
 					} else {
-						TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Chime);
+						TrainManager.AcknowledgeSafetySystem(Train, TrainManager.AcknowledgementType.Chime);
 						this.AtsChimeCancelPosition = double.PositiveInfinity;
 						this.AtsChimeCancelSection = -1;
 					}
-					if (Train.Specs.Security.Eb.BellState != TrainManager.SecurityState.Normal) {
+					if (Train.Specs.Safety.Eb.BellState != TrainManager.SafetyState.Normal) {
 						// eb
-						TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Eb);
+						TrainManager.AcknowledgeSafetySystem(Train, TrainManager.AcknowledgementType.Eb);
 					}
 					// do the ai
 					Train.Specs.CurrentConstSpeed = false;
 					TrainManager.ApplyHoldBrake(Train, false);
 					int stopIndex = Train.Station >= 0 ? GetStopIndex(Train.Station, Train.Cars.Length) : -1;
-					if (Train.CurrentSectionLimit == 0.0 & Train.Specs.Security.Mode != TrainManager.SecuritySystem.Atc) {
+					if (Train.CurrentSectionLimit == 0.0 & Train.Specs.Safety.Mode != TrainManager.SafetySystem.Atc) {
 						// passing red signal
 						TrainManager.ApplyEmergencyBrake(Train);
 						TrainManager.ApplyNotch(Train, -1, true, 1, true);
@@ -1155,13 +1161,13 @@ namespace OpenBve {
 					} else if (doorsopen | Train.StationState == TrainManager.TrainStopState.Boarding) {
 						// door opened or boarding at station
 						this.PowerNotchAtWhichWheelSlipIsObserved = Train.Specs.MaximumPowerNotch + 1;
-						if (Train.Station >= 0 && Stations[Train.Station].IsTerminalStation & Train == TrainManager.PlayerTrain) {
+						if (Train.Station >= 0 && Stations[Train.Station].IsTerminalStation && Train == TrainManager.PlayerTrain) {
 							// player's terminal station
 							TrainManager.ApplyReverser(Train, 0, false);
 							TrainManager.ApplyNotch(Train, -1, true, 1, true);
 							TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Service);
 							TrainManager.ApplyEmergencyBrake(Train);
-							Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.None;
+							Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.None;
 							CurrentInterval = 10.0;
 						} else {
 							TrainManager.ApplyNotch(Train, -1, true, 0, true);
@@ -1189,10 +1195,10 @@ namespace OpenBve {
 								}
 							} else if (Train.Station >= 0 & Train.StationState == TrainManager.TrainStopState.Boarding) {
 								// still boarding
-								if (Stations[Train.Station].SecuritySystem == SecuritySystem.Ats & Train.Specs.Security.Mode != TrainManager.SecuritySystem.AtsSN & Train.Specs.Security.Mode != TrainManager.SecuritySystem.AtsP) {
-									Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.AtsSN;
-								} else if (Stations[Train.Station].SecuritySystem == SecuritySystem.Atc & Train.Specs.Security.Mode != TrainManager.SecuritySystem.Atc & Train.Specs.Security.Atc.Available) {
-									Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.Atc;
+								if (Stations[Train.Station].SafetySystem == SafetySystem.Ats & Train.Specs.Safety.Mode != TrainManager.SafetySystem.AtsSn & Train.Specs.Safety.Mode != TrainManager.SafetySystem.AtsP) {
+									Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.AtsSn;
+								} else if (Stations[Train.Station].SafetySystem == SafetySystem.Atc & Train.Specs.Safety.Mode != TrainManager.SafetySystem.Atc & Train.Specs.Safety.Atc.Available) {
+									Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.Atc;
 								}
 							} else {
 								// not at station - close doors
@@ -1225,10 +1231,10 @@ namespace OpenBve {
 						// initialize
 						double spd = Train.Specs.CurrentAverageSpeed;
 						double acc = Train.Specs.CurrentAverageAcceleration;
-						double lim = double.PositiveInfinity;
-						if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.Atc) {
-							if (Train.Specs.Security.Atc.SpeedRestriction < lim) {
-								lim = Train.Specs.Security.Atc.SpeedRestriction;
+						double lim = PrecedingTrainSpeedLimit * 1.2;
+						if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.Atc) {
+							if (Train.Specs.Safety.Atc.SpeedRestriction < lim) {
+								lim = Train.Specs.Safety.Atc.SpeedRestriction;
 							}
 						} else {
 							if (Train.CurrentRouteLimit < lim) {
@@ -1245,17 +1251,17 @@ namespace OpenBve {
 							brakestart = lim;
 						} else {
 							lim *= this.PersonalitySpeedFactor;
-							if (spd < 8.33333333333333) {
-								powerstart = 0.666666666666667 * lim;
-								powerend = 0.833333333333333 * lim;
+							if (spd < 8.0) {
+								powerstart = 0.7 * lim;
+								powerend = 0.9 * lim;
 							} else {
-								powerstart = lim - 2.77777777777778;
-								powerend = lim - 1.38888888888889;
+								powerstart = lim - 2.5;
+								powerend = lim - 1.5;
 							}
 							if (this.BrakeMode) {
 								brakestart = powerend;
 							} else {
-								brakestart = lim + 0.555555555555556;
+								brakestart = lim + 0.5;
 							}
 						}
 						double dec = 0.0;
@@ -1299,7 +1305,7 @@ namespace OpenBve {
 										}
 									} else if (TrackManager.CurrentTrack.Elements[i].Events[j] is TrackManager.SectionChangeEvent) {
 										// section
-										if (Train.Specs.Security.Mode != TrainManager.SecuritySystem.Atc) {
+										if (Train.Specs.Safety.Mode != TrainManager.SafetySystem.Atc) {
 											TrackManager.SectionChangeEvent e = (TrackManager.SectionChangeEvent)TrackManager.CurrentTrack.Elements[i].Events[j];
 											if (stp + e.TrackPositionDelta > tp) {
 												if (!Game.Sections[e.NextSectionIndex].Invisible & Game.Sections[e.NextSectionIndex].CurrentAspect >= 0) {
@@ -1309,19 +1315,19 @@ namespace OpenBve {
 														double edec;
 														if (elim == 0.0) {
 															double redstopdist;
-															if (Train.Station >= 0 & Train.StationState == TrainManager.TrainStopState.Completed & dist < 150.0) {
+															if (Train.Station >= 0 & Train.StationState == TrainManager.TrainStopState.Completed & dist < 120.0) {
 																dist = 1.0;
 																redstopdist = 25.0;
 															} else if (Train.Station >= 0 & Train.StationState == TrainManager.TrainStopState.Pending) {
 																redstopdist = 1.0;
 															} else if (spd > 6.94444444444444) {
 																redstopdist = 85.0;
+															} else if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.AtsP) {
+																redstopdist = 35.0;
+															} else if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.AtsSn) {
+																redstopdist = 25.0;
 															} else {
-																if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.AtsP) {
-																	redstopdist = 35.0;
-																} else {
-																	redstopdist = 25.0;
-																}
+																redstopdist = 15.0;
 															}
 															if (dist > redstopdist) {
 																edec = (spd * spd) / (2.0 * (dist - redstopdist));
@@ -1348,23 +1354,33 @@ namespace OpenBve {
 											if (s >= 0) {
 												double dist = Stations[e.StationIndex].Stops[s].TrackPosition - tp;
 												if (dist > -Stations[e.StationIndex].Stops[s].ForwardTolerance) {
-													double edec;
-													if (dist < 0.0 | dist < Stations[e.StationIndex].Stops[s].BackwardTolerance & spd < 0.277777777777778) {
-														edec = BrakeDeceleration;
-													} else {
-														if (spd > 4.16666666666667) {
-															if (dist > 15.0) {
+													double Tback = Stations[e.StationIndex].Stops[s].BackwardTolerance;
+													if (dist >= Tback) {
+														double edec;
+														if (spd > 2.0) {
+															if (dist > 25.0) {
 																dist -= 7.5;
-															} else if (dist > 0.0) {
-																dist *= 0.5;
 															}
+															edec = spd * spd / (2.0 * dist);
+															if (edec > dec) dec = edec;
 														}
-														edec = (spd * spd) / (2.0 * dist);
 													}
-													if (edec > dec) dec = edec;
 												}
 											}
 										}
+									} else if (TrackManager.CurrentTrack.Elements[i].Events[j] is TrackManager.TrackEndEvent) {
+										// track end
+										if (Train == TrainManager.PlayerTrain) {
+											TrackManager.TrackEndEvent e = (TrackManager.TrackEndEvent)TrackManager.CurrentTrack.Elements[i].Events[j];
+											double dist = stp + e.TrackPositionDelta - tp;
+											double edec;
+											if (dist >= 15.0) {
+												edec = spd * spd / (2.0 * dist);
+											} else {
+												edec = BrakeDeceleration;
+											}
+											if (edec > dec) dec = edec;
+										}
 									}
 								}
 							}
@@ -1378,13 +1394,9 @@ namespace OpenBve {
 									double edec;
 									if (dist < 0.0) {
 										edec = BrakeDeceleration;
-									} else if (dist < Stations[Train.Station].Stops[s].BackwardTolerance) {
-										if (spd > 2.77777777777778) {
-											if (dist > 15.0) {
-												dist -= 7.5;
-											} else if (dist > 0.0) {
-												dist *= 0.5;
-											}
+									} else if (dist < Stations[Train.Station].Stops[s].BackwardTolerance | spd > 0.5) {
+										if (dist > 25.0) {
+											dist -= 7.5;
 										}
 										edec = (spd * spd) / (2.0 * dist);
 									} else {
@@ -1394,18 +1406,18 @@ namespace OpenBve {
 								}
 							}
 						}
-						// handle the security system
-						if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.AtsSN) {
+						// handle the safety system
+						if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.AtsSn) {
 							// ats-s
-							if (Train.Specs.Security.State == TrainManager.SecurityState.Ringing) {
+							if (Train.Specs.Safety.State == TrainManager.SafetyState.Ringing) {
 								dec = BrakeDeceleration;
 							}
-						} else if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.AtsP) {
+						} else if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.AtsP) {
 							// ats-p
-							if (Train.Specs.Security.State == TrainManager.SecurityState.Service & !Train.Specs.Security.Ats.AtsPOverride) {
+							if (Train.Specs.Safety.State == TrainManager.SafetyState.Service & !Train.Specs.Safety.Ats.AtsPOverride) {
 								powerstart = spd - 1.0;
 								powerend = spd - 1.0;
-							} else if (Train.Specs.Security.State == TrainManager.SecurityState.Pattern & !Train.Specs.Security.Ats.AtsPOverride) {
+							} else if (Train.Specs.Safety.State == TrainManager.SafetyState.Pattern & !Train.Specs.Safety.Ats.AtsPOverride) {
 								if (spd > 6.94444444444444) {
 									powerstart = spd - 1.0;
 									powerend = spd - 1.0;
@@ -1535,7 +1547,7 @@ namespace OpenBve {
 				this.TimeLastProcessed = 0.0;
 				this.CurrentInterval = 1.0;
 			}
-			internal override void Trigger(TrainManager.Train Train) {
+			internal override void Trigger(TrainManager.Train Train, double TimeElapsed) {
 				if (SecondsSinceMidnight - TimeLastProcessed >= CurrentInterval) {
 					TimeLastProcessed = SecondsSinceMidnight;
 					CurrentInterval = 5.0;
@@ -1725,7 +1737,7 @@ namespace OpenBve {
 			internal string Text;
 		}
 		internal static PointOfInterest[] PointsOfInterest = new PointOfInterest[] { };
-		internal static bool ApplyPointOfView(int Value, bool Relative) {
+		internal static bool ApplyPointOfInterest(int Value, bool Relative) {
 			double t = 0.0;
 			int j = -1;
 			if (Relative) {
