@@ -101,7 +101,8 @@ namespace OpenBve {
 			World.CameraTrackFollower.CarIndex = -1;
 			World.CameraMode = World.CameraViewMode.Interior;
 			// load route
-			CsvRwRouteParser.ParseRoute(CurrentRouteFile, CurrentRouteEncoding, CurrentTrainFolder, ObjectFolder, SoundFolder, false);
+			bool IsRW = string.Equals(System.IO.Path.GetExtension(CurrentRouteFile), ".rw", StringComparison.OrdinalIgnoreCase);
+			CsvRwRouteParser.ParseRoute(CurrentRouteFile, IsRW, CurrentRouteEncoding, CurrentTrainFolder, ObjectFolder, SoundFolder, false);
 			System.Threading.Thread.Sleep(1); if (Cancel) return;
 			Game.CalculateSeaLevelConstants();
 			if (Game.BogusPretrainInstructions.Length != 0) {
@@ -196,9 +197,18 @@ namespace OpenBve {
 								TrainManager.Trains[k].Cars[i].Specs.DoorCloseFrequency = 0.2;
 							}
 						}
-						const double f = 0.1;
-						TrainManager.Trains[k].Cars[i].Specs.DoorOpenFrequency *= Math.Exp(f * (2.0 * Game.Generator.NextDouble() - 1.0));
-						TrainManager.Trains[k].Cars[i].Specs.DoorCloseFrequency *= Math.Exp(f * (2.0 * Game.Generator.NextDouble() - 1.0));
+						const double f = 0.015;
+						const double g = 2.75;
+						TrainManager.Trains[k].Cars[i].Specs.DoorOpenPitch = Math.Exp(f * Math.Tan(g * (Game.Generator.NextDouble() - 0.5)));
+						TrainManager.Trains[k].Cars[i].Specs.DoorClosePitch = Math.Exp(f * Math.Tan(g * (Game.Generator.NextDouble() - 0.5)));
+						TrainManager.Trains[k].Cars[i].Specs.DoorOpenFrequency /= TrainManager.Trains[k].Cars[i].Specs.DoorOpenPitch;
+						TrainManager.Trains[k].Cars[i].Specs.DoorCloseFrequency /= TrainManager.Trains[k].Cars[i].Specs.DoorClosePitch;
+						/* 
+						 * Remove the following two lines, then the pitch at which doors play 
+						 * takes their randomized opening and closing times into account.
+						 * */
+						TrainManager.Trains[k].Cars[i].Specs.DoorOpenPitch = 1.0;
+						TrainManager.Trains[k].Cars[i].Specs.DoorClosePitch = 1.0;
 					}
 				}
 				for (int i = 0; i < TrainManager.Trains[k].Cars.Length; i++) {
@@ -222,34 +232,19 @@ namespace OpenBve {
 					ExtensionsCfgParser.ParseExtensionsConfig(CurrentTrainFolder, CurrentTrainEncoding, out CarObjects, TrainManager.Trains[k]);
 					System.Threading.Thread.Sleep(1); if (Cancel) return;
 					for (int i = 0; i < TrainManager.Trains[k].Cars.Length; i++) {
-						if (CarObjects[i] == null & Interface.CurrentOptions.ShowDefaultExteriorObjects) {
-							// create default solid-color object
-							ObjectManager.StaticObject o = new ObjectManager.StaticObject();
-							o.Mesh.Vertices = new World.Vertex[8];
-							double sx = 0.5 * TrainManager.Trains[k].Cars[i].Width;
-							double sy = TrainManager.Trains[k].Cars[i].Height;
-							double sz = 0.5 * TrainManager.Trains[k].Cars[i].Length;
-							o.Mesh.Vertices[0].Coordinates = new World.Vector3D(sx, sy, -sz);
-							o.Mesh.Vertices[1].Coordinates = new World.Vector3D(sx, 0.0, -sz);
-							o.Mesh.Vertices[2].Coordinates = new World.Vector3D(-sx, 0.0, -sz);
-							o.Mesh.Vertices[3].Coordinates = new World.Vector3D(-sx, sy, -sz);
-							o.Mesh.Vertices[4].Coordinates = new World.Vector3D(sx, sy, sz);
-							o.Mesh.Vertices[5].Coordinates = new World.Vector3D(sx, 0.0, sz);
-							o.Mesh.Vertices[6].Coordinates = new World.Vector3D(-sx, 0.0, sz);
-							o.Mesh.Vertices[7].Coordinates = new World.Vector3D(-sx, sy, sz);
-							o.Mesh.Materials = new World.MeshMaterial[1];
-							o.Mesh.Materials[0] = new World.MeshMaterial();
-							o.Mesh.Materials[0].Color = new World.ColorRGBA(192, 192, 192, 255);
-							o.Mesh.Materials[0].DaytimeTextureIndex = -1;
-							o.Mesh.Materials[0].NighttimeTextureIndex = -1;
-							o.Mesh.Faces = new World.MeshFace[6];
-							o.Mesh.Faces[0] = new World.MeshFace(new int[] { 0, 1, 2, 3 });
-							o.Mesh.Faces[1] = new World.MeshFace(new int[] { 0, 4, 5, 1 });
-							o.Mesh.Faces[2] = new World.MeshFace(new int[] { 0, 3, 7, 4 });
-							o.Mesh.Faces[3] = new World.MeshFace(new int[] { 6, 5, 4, 7 });
-							o.Mesh.Faces[4] = new World.MeshFace(new int[] { 6, 7, 3, 2 });
-							o.Mesh.Faces[5] = new World.MeshFace(new int[] { 6, 2, 1, 5 });
-							CarObjects[i] = o;
+						if (CarObjects[i] == null) {
+							// load default exterior object
+							string file = Interface.GetCombinedFileName(Interface.GetDataFolder("Compatibility"), "exterior.csv");
+							ObjectManager.StaticObject so = ObjectManager.LoadStaticObject(file, System.Text.Encoding.UTF8, ObjectManager.ObjectLoadMode.Normal, false, false, false);
+							if (so == null) {
+								CarObjects[i] = null;
+							} else {
+								double sx = TrainManager.Trains[k].Cars[i].Width;
+								double sy = TrainManager.Trains[k].Cars[i].Height;
+								double sz = TrainManager.Trains[k].Cars[i].Length;
+								CsvB3dObjectParser.ApplyScale(so, sx, sy, sz);
+								CarObjects[i] = so;
+							}
 						}
 						if (CarObjects[i] != null) {
 							// add object
@@ -292,7 +287,7 @@ namespace OpenBve {
 				if (TrainManager.Trains[k] == TrainManager.PlayerTrain) {
 					TrainManager.Trains[k].TimetableDelta = 0.0;
 				} else if (TrainManager.Trains[k].State != TrainManager.TrainState.Bogus) {
-					TrainManager.Trains[k].AI = new Game.SimplisticHumanDriverAI(TrainManager.Trains[k]);
+					TrainManager.Trains[k].AI = new Game.SimpleHumanDriverAI(TrainManager.Trains[k]);
 					TrainManager.Trains[k].TimetableDelta = Game.PrecedingTrainTimeDeltas[k];
 					TrainManager.Trains[k].Specs.DoorOpenMode = TrainManager.DoorMode.Manual;
 					TrainManager.Trains[k].Specs.DoorCloseMode = TrainManager.DoorMode.Manual;
@@ -312,7 +307,11 @@ namespace OpenBve {
 				if (Game.Stations[i].StopMode == Game.StationStopMode.AllStop | Game.Stations[i].StopMode == Game.StationStopMode.PlayerStop & Game.Stations[i].Stops.Length != 0) {
 					PlayerFirstStationIndex = i;
 					int s = Game.GetStopIndex(i, TrainManager.PlayerTrain.Cars.Length);
-					PlayerFirstStationPosition = Game.Stations[i].Stops[s].TrackPosition;
+					if (s >= 0) {
+						PlayerFirstStationPosition = Game.Stations[i].Stops[s].TrackPosition;
+					} else {
+						PlayerFirstStationPosition = Game.Stations[i].DefaultTrackPosition;
+					}
 					if (Game.Stations[i].ArrivalTime < 0.0) {
 						if (Game.Stations[i].DepartureTime < 0.0) {
 							Game.SecondsSinceMidnight = 0.0;
@@ -335,7 +334,11 @@ namespace OpenBve {
 				if (Game.Stations[i].StopMode == Game.StationStopMode.AllStop | Game.Stations[i].StopMode == Game.StationStopMode.PlayerPass & Game.Stations[i].Stops.Length != 0) {
 					OtherFirstStationIndex = i;
 					int s = Game.GetStopIndex(i, TrainManager.PlayerTrain.Cars.Length);
-					OtherFirstStationPosition = Game.Stations[i].Stops[s].TrackPosition;
+					if (s >= 0) {
+						OtherFirstStationPosition = Game.Stations[i].Stops[s].TrackPosition;
+					} else {
+						OtherFirstStationPosition = Game.Stations[i].DefaultTrackPosition;
+					}
 					if (Game.Stations[i].ArrivalTime < 0.0) {
 						if (Game.Stations[i].DepartureTime < 0.0) {
 							OtherFirstStationTime = 0.0;
@@ -414,15 +417,8 @@ namespace OpenBve {
 				} else {
 					p = OtherFirstStationPosition;
 				}
-				if (TrainManager.Trains[i].Cars[0].FrontAxle.Follower.TrackPosition < 0.1) {
-					double d = 0.1 - TrainManager.Trains[i].Cars[0].FrontAxle.Follower.TrackPosition;
-					for (int j = 0; j < TrainManager.Trains[i].Cars.Length; j++) {
-						TrainManager.MoveCar(TrainManager.Trains[i], j, p + d, 0.01);
-					}
-				} else {
-					for (int j = 0; j < TrainManager.Trains[i].Cars.Length; j++) {
-						TrainManager.MoveCar(TrainManager.Trains[i], j, p, 0.01);
-					}
+				for (int j = 0; j < TrainManager.Trains[i].Cars.Length; j++) {
+					TrainManager.MoveCar(TrainManager.Trains[i], j, p, 0.01);
 				}
 			}
 			// time table
@@ -432,6 +428,9 @@ namespace OpenBve {
 			}
 			// initialize camera
 			System.Threading.Thread.Sleep(1); if (Cancel) return;
+			if (World.CameraRestriction == World.CameraRestrictionMode.NotAvailable) {
+				World.CameraMode = World.CameraViewMode.InteriorLookAhead;
+			}
 			TrainManager.UpdateCamera(TrainManager.PlayerTrain);
 			TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, -1.0, true, false);
 			ObjectManager.UpdateVisibility(World.CameraTrackFollower.TrackPosition + World.CameraCurrentAlignment.Position.Z);
