@@ -59,7 +59,7 @@ namespace OpenBve {
 			World.InitializeCameraRestriction();
 			while (true) {
 				#if DEBUG
-				debugCheckError("MainLoop");
+				CheckForOpenGlError("MainLoop");
 				#endif
 				// timer
 				double TimeElapsed;
@@ -148,6 +148,8 @@ namespace OpenBve {
 					System.Threading.Thread.Sleep(10);
 				}
 			}
+			// finish
+			
 			try {
 				Interface.SaveLogs();
 			} catch { }
@@ -313,6 +315,7 @@ namespace OpenBve {
 					case Sdl.SDL_MOUSEBUTTONDOWN:
 						// mouse button down
 						if (Event.button.button == Sdl.SDL_BUTTON_RIGHT) {
+							// mouse grab
 							World.MouseGrabEnabled = !World.MouseGrabEnabled;
 							if (World.MouseGrabEnabled) {
 								World.MouseGrabTarget = new World.Vector2D(0.0, 0.0);
@@ -326,7 +329,9 @@ namespace OpenBve {
 						break;
 					case Sdl.SDL_MOUSEMOTION:
 						// mouse motion
-						if (World.MouseGrabEnabled) {
+						if (World.MouseGrabIgnoreOnce) {
+							World.MouseGrabIgnoreOnce = false;
+						} else if (World.MouseGrabEnabled) {
 							World.MouseGrabTarget = new World.Vector2D((double)Event.motion.xrel, (double)Event.motion.yrel);
 						}
 						break;
@@ -407,11 +412,11 @@ namespace OpenBve {
 											Game.MenuEntry[] a = Game.CurrentMenu;
 											int j = Game.CurrentMenuSelection.Length - 1;
 											{
-												int k = 0; while (k < j) {
+												int k = 0;
+												while (k < j) {
 													Game.MenuSubmenu b = a[Game.CurrentMenuSelection[k]] as Game.MenuSubmenu;
 													if (b == null) break;
 													a = b.Entries; k++;
-
 												}
 											}
 											if (a[Game.CurrentMenuSelection[j]] is Game.MenuCommand) {
@@ -436,13 +441,23 @@ namespace OpenBve {
 																for (int h = 0; h < TrainManager.PlayerTrain.Cars.Length; h++) {
 																	TrainManager.PlayerTrain.Cars[h].Specs.CurrentSpeed = 0.0;
 																}
-																double p = Game.Stations[k].Stops[t].TrackPosition;
-																double d = p - TrainManager.PlayerTrain.Cars[0].FrontAxle.Follower.TrackPosition;
+																double d = Game.Stations[k].Stops[t].TrackPosition - TrainManager.PlayerTrain.Cars[0].FrontAxle.Follower.TrackPosition + TrainManager.PlayerTrain.Cars[0].FrontAxlePosition - 0.5 * TrainManager.PlayerTrain.Cars[0].Length;
 																TrackManager.SuppressSoundEvents = true;
-																while (Math.Abs(d) > 1.0) {
+																while (d != 0.0) {
+																	double x;
+																	if (Math.Abs(d) > 1.0) {
+																		x = (double)Math.Sign(d);
+																	} else {
+																		x = d;
+																	}
 																	for (int h = 0; h < TrainManager.PlayerTrain.Cars.Length; h++) {
-																		TrainManager.MoveCar(TrainManager.PlayerTrain, h, (double)Math.Sign(d), 0.0);
-																	} d -= (double)Math.Sign(d);
+																		TrainManager.MoveCar(TrainManager.PlayerTrain, h, x, 0.0);
+																	}
+																	if (Math.Abs(d) >= 1.0) {
+																		d -= x;
+																	} else {
+																		break;
+																	}
 																}
 																TrackManager.SuppressSoundEvents = false;
 																if (TrainManager.PlayerTrain.Specs.CurrentEmergencyBrake.Driver) {
@@ -469,7 +484,8 @@ namespace OpenBve {
 																Game.Messages = new Game.Message[] { };
 																ObjectManager.UpdateAnimatedWorldObjects(TimeElapsed, true);
 															}
-														} break;
+														}
+														break;
 													case Game.MenuTag.ExitToMainMenu:
 														Program.RestartProcessArguments = Interface.CurrentOptions.GameMode == Interface.GameMode.Arcade ? "/review" : "";
 														Quit = true;
@@ -485,10 +501,21 @@ namespace OpenBve {
 												int n = Game.CurrentMenuSelection.Length;
 												Array.Resize<int>(ref Game.CurrentMenuSelection, n + 1);
 												Array.Resize<double>(ref Game.CurrentMenuOffsets, n + 1);
-												int k; for (k = 0; k < b.Entries.Length; k++) {
-													if (!(b.Entries[k] is Game.MenuCaption)) break;
+												/* Select the first non-caption entry. */
+												int selection;
+												for (selection = 0; selection < b.Entries.Length; selection++) {
+													if (!(b.Entries[selection] is Game.MenuCaption)) break;
 												}
-												Game.CurrentMenuSelection[n] = k < b.Entries.Length ? k : 0;
+												/* Select the current/next station if this menu has stations in it. */
+												for (int k = selection + 1; k < b.Entries.Length; k++) {
+													Game.MenuCommand c = b.Entries[k] as Game.MenuCommand;
+													if (c != null && c.Tag == Game.MenuTag.JumpToStation) {
+														if (c.Data <= TrainManager.PlayerTrain.CurrentSectionIndex) {
+															selection = k;
+														}
+													}
+												}
+												Game.CurrentMenuSelection[n] = selection < b.Entries.Length ? selection : 0;
 												Game.CurrentMenuOffsets[n] = double.NegativeInfinity;
 												a = b.Entries;
 												for (int h = 0; h < a.Length; h++) {
@@ -751,7 +778,7 @@ namespace OpenBve {
 									case Interface.Command.CameraInterior:
 										// camera: interior
 										SaveCameraSettings();
-										if (World.CameraMode == World.CameraViewMode.Interior & World.CameraRestriction == World.CameraRestrictionMode.NotAvailable) {
+										if (World.CameraMode != World.CameraViewMode.InteriorLookAhead & World.CameraRestriction == World.CameraRestrictionMode.NotAvailable) {
 											World.CameraMode = World.CameraViewMode.InteriorLookAhead;
 											Game.AddMessage(Interface.GetInterfaceString("notification_interior_lookahead"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
 										} else {
@@ -840,7 +867,7 @@ namespace OpenBve {
 										} break;
 									case Interface.Command.CameraPreviousPOI:
 										// camera: previous poi
-										if (Game.ApplyPointOfView(-1, true)) {
+										if (Game.ApplyPointOfInterest(-1, true)) {
 											if (World.CameraMode != World.CameraViewMode.Track & World.CameraMode != World.CameraViewMode.FlyBy & World.CameraMode != World.CameraViewMode.FlyByZooming) {
 												World.CameraMode = World.CameraViewMode.Track;
 												Game.AddMessage(Interface.GetInterfaceString("notification_track"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
@@ -871,7 +898,7 @@ namespace OpenBve {
 										} break;
 									case Interface.Command.CameraNextPOI:
 										// camera: next poi
-										if (Game.ApplyPointOfView(1, true)) {
+										if (Game.ApplyPointOfInterest(1, true)) {
 											if (World.CameraMode != World.CameraViewMode.Track & World.CameraMode != World.CameraViewMode.FlyBy & World.CameraMode != World.CameraViewMode.FlyByZooming) {
 												World.CameraMode = World.CameraViewMode.Track;
 												Game.AddMessage(Interface.GetInterfaceString("notification_track"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
@@ -912,10 +939,10 @@ namespace OpenBve {
 											TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, TrainManager.PlayerTrain.Cars[0].FrontAxle.Follower.TrackPosition, true, false);
 										} else if (World.CameraMode == World.CameraViewMode.FlyBy | World.CameraMode == World.CameraViewMode.FlyByZooming) {
 											if (TrainManager.PlayerTrain.Specs.CurrentAverageSpeed >= 0.0) {
-												double d = 30.0 + TrainManager.PlayerTrain.Specs.CurrentAverageSpeed;
+												double d = 30.0 + 4.0 * TrainManager.PlayerTrain.Specs.CurrentAverageSpeed;
 												TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, TrainManager.PlayerTrain.Cars[0].FrontAxle.Follower.TrackPosition + d, true, false);
 											} else {
-												double d = 30.0 - TrainManager.PlayerTrain.Specs.CurrentAverageSpeed;
+												double d = 30.0 - 4.0 * TrainManager.PlayerTrain.Specs.CurrentAverageSpeed;
 												TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, TrainManager.PlayerTrain.Cars[TrainManager.PlayerTrain.Cars.Length - 1].RearAxle.Follower.TrackPosition - d, true, false);
 											}
 										}
@@ -1195,71 +1222,71 @@ namespace OpenBve {
 										} break;
 									case Interface.Command.SecurityPower:
 										// security: power
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.None) {
-											if (TrainManager.PlayerTrain.Specs.Security.Ats.AtsAvailable) {
-												TrainManager.PlayerTrain.Specs.Security.ModeChange = TrainManager.SecuritySystem.AtsSN;
-											} else if (TrainManager.PlayerTrain.Specs.Security.Atc.Available) {
-												TrainManager.PlayerTrain.Specs.Security.ModeChange = TrainManager.SecuritySystem.Atc;
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.None) {
+											if (TrainManager.PlayerTrain.Specs.Safety.Ats.AtsAvailable) {
+												TrainManager.PlayerTrain.Specs.Safety.ModeChange = TrainManager.SafetySystem.AtsSn;
+											} else if (TrainManager.PlayerTrain.Specs.Safety.Atc.Available) {
+												TrainManager.PlayerTrain.Specs.Safety.ModeChange = TrainManager.SafetySystem.Atc;
 											}
 										} else {
-											TrainManager.PlayerTrain.Specs.Security.ModeChange = TrainManager.SecuritySystem.None;
+											TrainManager.PlayerTrain.Specs.Safety.ModeChange = TrainManager.SafetySystem.None;
 										} break;
 									case Interface.Command.SecurityS:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_S, true);
-										TrainManager.AcknowledgeSecuritySystem(TrainManager.PlayerTrain, TrainManager.AcknowledgementType.Alarm);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_S, true);
+										TrainManager.AcknowledgeSafetySystem(TrainManager.PlayerTrain, TrainManager.AcknowledgementType.Alarm);
 										break;
 									case Interface.Command.SecurityA1:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_A1, true);
-										TrainManager.AcknowledgeSecuritySystem(TrainManager.PlayerTrain, TrainManager.AcknowledgementType.Chime);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_A1, true);
+										TrainManager.AcknowledgeSafetySystem(TrainManager.PlayerTrain, TrainManager.AcknowledgementType.Chime);
 										break;
 									case Interface.Command.SecurityA2:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_A2, true);
-										TrainManager.AcknowledgeSecuritySystem(TrainManager.PlayerTrain, TrainManager.AcknowledgementType.Eb);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_A2, true);
+										TrainManager.AcknowledgeSafetySystem(TrainManager.PlayerTrain, TrainManager.AcknowledgementType.Eb);
 										break;
 									case Interface.Command.SecurityB1:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_B1, true);
-										TrainManager.AcknowledgeSecuritySystem(TrainManager.PlayerTrain, TrainManager.AcknowledgementType.Reset);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_B1, true);
+										TrainManager.AcknowledgeSafetySystem(TrainManager.PlayerTrain, TrainManager.AcknowledgementType.Reset);
 										break;
 									case Interface.Command.SecurityB2:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_B2, true);
-										TrainManager.AcknowledgeSecuritySystem(TrainManager.PlayerTrain, TrainManager.AcknowledgementType.Override);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_B2, true);
+										TrainManager.AcknowledgeSafetySystem(TrainManager.PlayerTrain, TrainManager.AcknowledgementType.Override);
 										break;
 									case Interface.Command.SecurityC1:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_C1, true);
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Atc & TrainManager.PlayerTrain.Specs.Security.Ats.AtsAvailable) {
-											TrainManager.PlayerTrain.Specs.Security.ModeChange = TrainManager.SecuritySystem.AtsSN;
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_C1, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Atc & TrainManager.PlayerTrain.Specs.Safety.Ats.AtsAvailable) {
+											TrainManager.PlayerTrain.Specs.Safety.ModeChange = TrainManager.SafetySystem.AtsSn;
 										} break;
 									case Interface.Command.SecurityC2:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_C2, true);
-										if (TrainManager.PlayerTrain.Specs.Security.Atc.Available & (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.AtsSN | TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.AtsP)) {
-											TrainManager.PlayerTrain.Specs.Security.ModeChange = TrainManager.SecuritySystem.Atc;
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_C2, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Atc.Available & (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.AtsSn | TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.AtsP)) {
+											TrainManager.PlayerTrain.Specs.Safety.ModeChange = TrainManager.SafetySystem.Atc;
 										} break;
 									case Interface.Command.SecurityD:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_D, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_D, true);
 										break;
 									case Interface.Command.SecurityE:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_E, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_E, true);
 										break;
 									case Interface.Command.SecurityF:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_F, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_F, true);
 										break;
 									case Interface.Command.SecurityG:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_G, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_G, true);
 										break;
 									case Interface.Command.SecurityH:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_H, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_H, true);
 										break;
 									case Interface.Command.SecurityI:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_I, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_I, true);
 										break;
 									case Interface.Command.SecurityJ:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_J, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_J, true);
 										break;
 									case Interface.Command.SecurityK:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_K, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_K, true);
 										break;
 									case Interface.Command.SecurityL:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_L, true);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_L, true);
 										break;
 									case Interface.Command.TimetableToggle:
 										// option: timetable
@@ -1293,8 +1320,8 @@ namespace OpenBve {
 											Game.AddMessage(Interface.GetInterfaceString("notification_notavailableexpert"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 5.0);
 										} else {
 											if (TrainManager.PlayerTrain.AI == null) {
-												TrainManager.PlayerTrain.AI = new Game.SimplisticHumanDriverAI(TrainManager.PlayerTrain);
-												if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) {
+												TrainManager.PlayerTrain.AI = new Game.SimpleHumanDriverAI(TrainManager.PlayerTrain);
+												if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) {
 													Game.AddMessage(Interface.GetInterfaceString("notification_aiunable"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 10.0);
 												}
 											} else {
@@ -1379,52 +1406,52 @@ namespace OpenBve {
 								Interface.CurrentControls[i].DigitalState = Interface.DigitalControlState.ReleasedAcknowledged;
 								switch (Interface.CurrentControls[i].Command) {
 									case Interface.Command.SecurityS:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_S, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_S, false);
 										break;
 									case Interface.Command.SecurityA1:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_A1, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_A1, false);
 										break;
 									case Interface.Command.SecurityA2:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_A2, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_A2, false);
 										break;
 									case Interface.Command.SecurityB1:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_B1, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_B1, false);
 										break;
 									case Interface.Command.SecurityB2:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_B2, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_B2, false);
 										break;
 									case Interface.Command.SecurityC1:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_C1, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_C1, false);
 										break;
 									case Interface.Command.SecurityC2:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_C2, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_C2, false);
 										break;
 									case Interface.Command.SecurityD:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_D, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_D, false);
 										break;
 									case Interface.Command.SecurityE:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_E, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_E, false);
 										break;
 									case Interface.Command.SecurityF:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_F, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_F, false);
 										break;
 									case Interface.Command.SecurityG:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_G, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_G, false);
 										break;
 									case Interface.Command.SecurityH:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_H, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_H, false);
 										break;
 									case Interface.Command.SecurityI:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_I, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_I, false);
 										break;
 									case Interface.Command.SecurityJ:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_J, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_J, false);
 										break;
 									case Interface.Command.SecurityK:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_K, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_K, false);
 										break;
 									case Interface.Command.SecurityL:
-										if (TrainManager.PlayerTrain.Specs.Security.Mode == TrainManager.SecuritySystem.Bve4Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_L, false);
+										if (TrainManager.PlayerTrain.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) PluginManager.UpdateKey(PluginManager.ATS_KEY_L, false);
 										break;
 								}
 							}
@@ -1480,6 +1507,9 @@ namespace OpenBve {
 		// toggle fullscreen
 		internal static void ToggleFullscreen() {
 			Interface.CurrentOptions.FullscreenMode = !Interface.CurrentOptions.FullscreenMode;
+			if (World.MouseGrabEnabled) {
+				Sdl.SDL_WM_GrabInput(Sdl.SDL_GRAB_OFF);
+			}
 			Gl.glDisable(Gl.GL_FOG); Renderer.FogEnabled = false;
 			Gl.glDisable(Gl.GL_LIGHTING); Renderer.LightingEnabled = false;
 			TextureManager.UnuseAllTextures();
@@ -1498,7 +1528,17 @@ namespace OpenBve {
 			InitializeMotionBlur();
 			Timetable.CreateTimetable();
 			Timetable.UpdateCustomTimetable(-1, -1);
+			if (World.MouseGrabEnabled) {
+				Sdl.SDL_WM_GrabInput(Sdl.SDL_GRAB_ON);
+			}
+			World.MouseGrabTarget = new World.Vector2D(0.0, 0.0);
+			World.MouseGrabIgnoreOnce = true;
 			World.InitializeCameraRestriction();
+			if (Renderer.OptionBackfaceCulling) {
+				Gl.glEnable(Gl.GL_CULL_FACE);
+			} else {
+				Gl.glDisable(Gl.GL_CULL_FACE);
+			}
 		}
 
 		// update viewport
@@ -1555,7 +1595,7 @@ namespace OpenBve {
 		
 		#if DEBUG
 		// check error
-		private static void debugCheckError(string Location) {
+		private static void CheckForOpenGlError(string Location) {
 			int error = Gl.glGetError();
 			if (error != Gl.GL_NO_ERROR) {
 				string message = Location + ": ";
@@ -1585,8 +1625,7 @@ namespace OpenBve {
 						message += error.ToString();
 						break;
 				}
-				System.Windows.Forms.MessageBox.Show(message);
-				Quit = true;
+				throw new InvalidOperationException(message);
 			}
 		}
 		#endif
