@@ -813,6 +813,12 @@ namespace OpenBve {
 			// transform subject
 			string Code;
 			switch (Subject.ToLowerInvariant()) {
+				case "acc":
+					Code = "acceleration";
+					break;
+				case "motor":
+					Code = "accelerationmotor";
+					break;
 				case "true":
 					Code = "1";
 					break;
@@ -885,6 +891,9 @@ namespace OpenBve {
 								if (n >= 0 & n < Train.Cars.Length) {
 									Code = n.ToString(Culture) + " leftdoorsindex ceiling";
 									unsupported = false;
+								} else {
+									Code = "2";
+									unsupported = false;
 								}
 							}
 						} else if (Subject.StartsWith("doorr", StringComparison.OrdinalIgnoreCase)) {
@@ -893,6 +902,9 @@ namespace OpenBve {
 								if (n >= 0 & n < Train.Cars.Length) {
 									Code = n.ToString(Culture) + " rightdoorsindex ceiling";
 									unsupported = false;
+								} else {
+									Code = "2";
+									unsupported = false;
 								}
 							}
 						}
