@@ -12,6 +12,8 @@ namespace OpenBve {
 		internal static Platform CurrentPlatform = Platform.Windows;
 		internal static bool CurrentlyRunOnMono = false;
 		internal static bool UseFilesystemHierarchyStandard = false;
+		internal enum ProgramType { OpenBve, RouteViewer, ObjectViewer, Other };
+		internal const ProgramType CurrentProgramType = ProgramType.OpenBve;
 		private static bool SdlWindowCreated = false;
 
 		// main
@@ -66,6 +68,7 @@ namespace OpenBve {
 			if(SdlWindowCreated & Interface.CurrentOptions.FullscreenMode) {
 				Sdl.SDL_SetVideoMode(Interface.CurrentOptions.WindowWidth, Interface.CurrentOptions.WindowHeight, 32, Sdl.SDL_OPENGL | Sdl.SDL_DOUBLEBUF);
 			}
+			Renderer.Deinitialize();
 			TextureManager.UnuseAllTextures();
 			Asynchronous.Deinitialize();
 			PluginManager.UnloadPlugin();
@@ -116,8 +119,8 @@ namespace OpenBve {
 				for (int i = 0; i < n; i++) {
 					Interface.CurrentJoysticks[i].SdlHandle = Sdl.SDL_JoystickOpen(i);
 					if (CurrentPlatform == Platform.Windows) {
-						/// string returned is ascii packed in utf-16 (2 chars per codepoint)
 						string s = Sdl.SDL_JoystickName(i);
+						/* string returned is ascii packed in utf-16 (2 chars per codepoint) */
 						System.Text.StringBuilder t = new System.Text.StringBuilder(s.Length << 1);
 						for (int k = 0; k < s.Length; k++) {
 							int a = (int)s[k];
@@ -187,7 +190,8 @@ namespace OpenBve {
 			}
 			// route provided but no train
 			if (Result.RouteFile != null & Result.TrainFolder == null) {
-				CsvRwRouteParser.ParseRoute(Result.RouteFile, Result.RouteEncoding, null, null, null, true);
+				bool IsRW = string.Equals(System.IO.Path.GetExtension(Result.RouteFile), ".rw", StringComparison.OrdinalIgnoreCase);
+				CsvRwRouteParser.ParseRoute(Result.RouteFile, IsRW, Result.RouteEncoding, null, null, null, true);
 				if (Game.TrainName != null && Game.TrainName.Length != 0) {
 					string Folder = System.IO.Path.GetDirectoryName(Result.RouteFile);
 					while (true) {
@@ -233,7 +237,8 @@ namespace OpenBve {
 			Renderer.ScreenWidth = Width;
 			Renderer.ScreenHeight = Height;
 			World.AspectRatio = (double)Renderer.ScreenWidth / (double)Renderer.ScreenHeight;
-			World.VerticalViewingAngle = 45.0 * 0.0174532925199433;
+			const double degree = 0.0174532925199433;
+			World.VerticalViewingAngle = 45.0 * degree;
 			World.HorizontalViewingAngle = 2.0 * Math.Atan(Math.Tan(0.5 * World.VerticalViewingAngle) * World.AspectRatio);
 			World.OriginalVerticalViewingAngle = World.VerticalViewingAngle;
 			World.ExtraViewingDistance = 50.0;
