@@ -463,6 +463,7 @@ namespace OpenBve {
 		
 		// mouse grab
 		internal static bool MouseGrabEnabled = false;
+		internal static bool MouseGrabIgnoreOnce = false;
 		internal static Vector2D MouseGrabTarget = new Vector2D(0.0, 0.0);
 		internal static void UpdateMouseGrab(double TimeElapsed) {
 			if (MouseGrabEnabled) {
@@ -571,6 +572,7 @@ namespace OpenBve {
 				Source = Target;
 				return true;
 			} else {
+				double best = Source;
 				const int Precision = 8;
 				double a = Source;
 				double b = Target;
@@ -587,12 +589,13 @@ namespace OpenBve {
 						q = PerformCameraRestrictionTest();
 						if (q) {
 							a = x;
+							best = x;
 						} else {
 							b = x;
 						}
 						x = 0.5 * (a + b);
 					}
-					Source = a;
+					Source = best;
 					if (Zoom) ApplyZoom();
 					return q;
 				}
@@ -629,7 +632,7 @@ namespace OpenBve {
 		internal static void UpdateAbsoluteCamera(double TimeElapsed) {
 			// zoom
 			double zm = World.CameraCurrentAlignment.Zoom;
-			AdjustAlignment(ref World.CameraCurrentAlignment.Zoom, World.CameraAlignmentDirection.Zoom, ref World.CameraAlignmentSpeed.Zoom, TimeElapsed, World.CameraAlignmentSpeed.Zoom != 0.0);
+			AdjustAlignment(ref World.CameraCurrentAlignment.Zoom, World.CameraAlignmentDirection.Zoom, ref World.CameraAlignmentSpeed.Zoom, TimeElapsed, true);
 			if (zm != World.CameraCurrentAlignment.Zoom) {
 				ApplyZoom();
 			}
@@ -767,7 +770,13 @@ namespace OpenBve {
 						double ty = (rz * (dy * sx - dx * sy) + ry * (-dz * sx + dx * sz) + rx * (dz * sy - dy * sz)) * t;
 						double tz = (rz * (sy * ux - sx * uy) + ry * (-sz * ux + sx * uz) + rx * (sz * uy - sy * uz)) * t;
 						lookaheadYaw = tx * tz != 0.0 ? Math.Atan2(tx, tz) : 0.0;
-						lookaheadPitch = Math.Asin(ty);
+						if (ty < -1.0) {
+							lookaheadPitch = -0.5 * Math.PI;
+						} else if (ty > 1.0) {
+							lookaheadPitch = 0.5 * Math.PI;
+						} else {
+							lookaheadPitch = Math.Asin(ty);
+						}
 					} else {
 						lookaheadYaw = 0.0;
 						lookaheadPitch = 0.0;
@@ -929,28 +938,30 @@ namespace OpenBve {
 			AdjustAlignment(ref Source, Direction, ref Speed, TimeElapsed, false);
 		}
 		private static void AdjustAlignment(ref double Source, double Direction, ref double Speed, double TimeElapsed, bool Zoom) {
-			if (TimeElapsed > 0.0) {
-				if (Direction == 0.0) {
-					double d = (0.025 + 5.0 * Math.Abs(Speed)) * TimeElapsed;
-					if (Speed >= -d & Speed <= d) {
-						Speed = 0.0;
+			if (Direction != 0.0 | Speed != 0.0) {
+				if (TimeElapsed > 0.0) {
+					if (Direction == 0.0) {
+						double d = (0.025 + 5.0 * Math.Abs(Speed)) * TimeElapsed;
+						if (Speed >= -d & Speed <= d) {
+							Speed = 0.0;
+						} else {
+							Speed -= (double)Math.Sign(Speed) * d;
+						}
 					} else {
-						Speed -= (double)Math.Sign(Speed) * d;
+						double t = Math.Abs(Direction);
+						double d = ((1.15 - 1.0 / (1.0 + 0.025 * Math.Abs(Speed)))) * TimeElapsed;
+						Speed += Direction * d;
+						if (Speed < -t) {
+							Speed = -t;
+						} else if (Speed > t) {
+							Speed = t;
+						}
 					}
-				} else {
-					double t = Math.Abs(Direction);
-					double d = ((1.15 - 1.0 / (1.0 + 0.025 * Math.Abs(Speed)))) * TimeElapsed;
-					Speed += Direction * d;
-					if (Speed < -t) {
-						Speed = -t;
-					} else if (Speed > t) {
-						Speed = t;
+					double x = Source + Speed * TimeElapsed;
+					if (!PerformProgressiveAdjustmentForCameraRestriction(ref Source, x, Zoom)) {
+						Speed = 0.0;
 					}
 				}
-				double x = Source + Speed * TimeElapsed;
-				if (!PerformProgressiveAdjustmentForCameraRestriction(ref Source, x, Zoom)) {
-					Speed = 0.0;
-				}
 			}
 		}
 		private static void ApplyZoom() {
